@@ -3,16 +3,28 @@
 # AuthDriver.pm:
 # External authenticator for use with tpop3d's auth-other.
 #
-# Copyright (c) 2001 Chris Lightfoot. All rights reserved.
+# Copyright (c) 2001 Chris Lightfoot.
 #
-# $Id: AuthDriver.pm,v 1.1 2001/04/23 23:45:40 chris Exp $
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2, or (at your option)
+# any later version.
 #
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software Foundation,
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+
 
 # POD begins
 
 =head1 NAME
 
-TPOP3D::AuthDriver - library to interface to tpop3d's auth-other
+TPOP3D::AuthDriver - library to interface to tpop3d's auth-other and auth-perl
 
 =head1 SYNOPSIS
 
@@ -31,7 +43,7 @@ TPOP3D::AuthDriver - library to interface to tpop3d's auth-other
   
   sub pass($$) {
     my ($self, $req) = @_;
-    if ($req->{'user'} eq "fred" and $req->{pass} eq "secret") {
+    if ($req->{user} eq "fred" and $req->{pass} eq "secret") {
       # Success.
       return { 'result' => 'YES',
                'logmsg' => 'authenticated Fred',
@@ -41,16 +53,30 @@ TPOP3D::AuthDriver - library to interface to tpop3d's auth-other
                'mailbox' => '/var/spool/mail/fred' };
     } else {
       # Failure.
-      return { 'result' => 'NO', 'logmsg' => 'authentication failed' };
+      return { 'result' => 'NO',
+               'logmsg' => 'authentication failed' };
     }
   }
 
   1;
 
-  package Main;
+  package main;
 
   $auth = new FredAuthDriver;
-  $auth->run();
+
+  # function for auth-perl compatibility
+  sub passauth ($) {
+    return $auth->pass($_[0]);
+  }
+  
+  # run in auth-other mode
+  if (defined($ENV{TPOP3D_CONTEXT})
+      and $ENV{TPOP3D_CONTEXT eq 'auth_other') {
+    $auth->run();
+    exit 0;
+  }
+
+  1;
 
 =head1 DESCRIPTION
 
@@ -67,9 +93,7 @@ virtual-domains configurations.
 
 package TPOP3D::AuthDriver;
 
-use MD5;
-
-$VERSION = '0.1';
+$VERSION = '0.2';
 
 =item new
 
@@ -95,30 +119,27 @@ SIGTERM; on receiving this signal, the method will return.
 =cut
 sub run ($) {
     my $self = shift;
-    my ($buffer, $offset);
+    my ($buffer);
     local %SIG;
     $SIG{TERM} = sub { $self->{foad} = 1; };
 
     $self->start();
 
     $buffer = '';
-    $offset = 0;
     do {
         my $readfds = '';
         vec($readfds, fileno(STDIN), 1) = 1;
         if (select($readfds, undef, undef, 0.1) == 1) {
-            my $i = sysread(STDIN, $buffer, 4096, $offset);
-            if ($i > 0) {
-                $offset += $i;
-            } else {
+            my $i = sysread(STDIN, $buffer, 4096, length($buffer));
+            if ($i <= 0) {
                 $self->{foad} = 1;
             }
 
             # Now see whether the first part of the buffer has the right
             # structure:
-            if ($buffer =~ /((?:(?:[^\0]+\0){2})+)\0/) {
+            if ($buffer =~ /^((?:(?:[^\0]+\0){2})+)\0/) {
                 my $packet = $1;
-                $buffer = substr($buffer, length($packet));
+                $buffer = substr($buffer, length($packet) + 1);
                 last unless ($self->process_packet($packet));
             }
         }
@@ -135,7 +156,8 @@ sub process_packet ($$) {
     my %hash = split("\0", $packet);
     
     $res = $self->apop(\%hash) if ($hash{method} eq 'APOP');
-    $res = $self->pass(\%hash) if ($hash{method} eq 'USER');
+    $res = $self->pass(\%hash) if ($hash{method} eq 'PASS');
+    $res = $self->onlogin(\%hash) if ($hash{method} eq 'ONLOGIN');
 
     if (!defined($res)) {
         # OK, we didn't handle it; perhaps we are chained to another handler?
@@ -186,56 +208,46 @@ This method is called by run() when a request for APOP authentication is
 received. REQUEST is a reference to a hash of the parameters supplied by the
 server, including
 
-=over 4
+  timestamp
+    server's RFC1939 timestamp
 
-=item timestamp
+  user
+    client's supplied username
 
-server's RFC1939 timestamp
+  local_part
+  domain
+    local-part and domain derived from client's username;
+    may not be present
 
-=item user
+  digest
+    client's supplied digest, in hex
 
-client's supplied username
-
-=item digest
-
-client's supplied digest, in hex
-
-=back
+  clienthost
+    IP number of the client host
 
 
 It should return a reference to a hash containing the following keys:
 
-=over 4
+  result
+    if authentication was successful, `YES'; otherwise `NO'
 
-=item result
+  uid
+    username/uid with which to access mailspool
 
-if authentication was successful, `YES'; otherwise `NO'
+  gid
+    groupname/gid with which to access mailspool
 
-=item uid
+  domain
+    (optional) domain in which the user has been authenticated
 
-username/uid with which to access mailspool
+  mailbox
+    (optional) location of mailbox
 
-=item gid
+  mboxtype
+    (optional) name of mailbox driver
 
-groupname/gid with which to access mailspool
-
-=item domain
-
-(optional) domain in which the user has been authenticated
-
-=item mailbox
-
-(optional) location of mailbox
-
-=item mboxtype
-
-(optional) name of mailbox driver
-
-=item logmsg
-
-(optional) message to log
-
-=back 4
+  logmsg
+    (optional) message to log
 
 Note that if you do not supply a value for C<mailspool>, then the mailspool
 name will be determined from the `mailbox:' and `auth-other-mailbox:'
@@ -276,17 +288,19 @@ This method is called by run() when a request for USER/PASS authentication is
 received. REQUEST is a reference to a hash of the parameters supplied by the
 server, including
 
-=over
+  user
+    client's supplied username
 
-=item user
+  local_part
+  domain
+    local-part and domain derived from client's username;
+    may not be present
 
-client's supplied username
+  pass
+    client's supplied password
 
-=item pass
-
-client's supplied password
-
-=back
+  clienthost
+    IP number of the client host
 
 It should return a reference to a hash, as described for the C<apop> method
 above. You should override this to perform USER/PASS authentication, if you
@@ -299,14 +313,42 @@ sub pass ($$) {
     return { 'result' => 'NO' };    # default
 }
 
+=item onlogin REQUEST
+
+I<Instance method.>
+This method is called by run() when a packet describing a successful login is
+received. REQUEST is a reference to a hash of the parameters supplied by the
+server, including
+
+  local_part
+    client's supplied username or local-part
+
+  domain
+    domain in which client is authenticated
+
+  clienthost
+    IP number of the client host
+
+It should return either an empty hash reference, or one containing only the
+logmsg member.
+
+=cut
+
+sub onlogin ($$) {
+    my ($self, $req) = @_;
+    return { };
+}
+
 1;
 
 __END__
 
+=back
+
 =head1 COPYING
 
-Copyright (c) 2001 Chris Lightfoot, <chris@ex-parrot.com>
-F<http://www.ex-parrot.com/~chris/tpop3d>
+Copyright (c) 2001-2 Chris Lightfoot, <chris@ex-parrot.com>
+F<http://www.ex-parrot.com/~chris/tpop3d/>
 
 This program is free software; you can redistribute and/or modify it under the
 same terms as Perl itself.
@@ -317,8 +359,8 @@ None yet; please send me information when you find them.
 
 =head1 VERSION
 
-$Id: AuthDriver.pm,v 1.1 2001/04/23 23:45:40 chris Exp $
+$Id$
 
 =head1 SEE ALSO
 
-F<tpop3d>, F<RFC1939>, F<http://www.ex-parrot.com/~chris/tpop3d/>
+L<tpop3d(8)>, L<tpop3d.conf(5)>, F<RFC1939>, F<http://www.ex-parrot.com/~chris/tpop3d/>
