@@ -3,7 +3,21 @@
  * Authenticate users against a LDAP server.
  *
  * designed for tpop3d by Sebastien THOMAS (prune@lecentre.net) - Mad Cow tribe
- * Copyright (c) 2002 Sebastien Thomas, Chris Lightfoot. All rights reserved.
+ * Copyright (c) 2002 Sebastien Thomas, Chris Lightfoot.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  * 
  */
 
@@ -12,10 +26,14 @@
 #endif /* HAVE_CONFIG_H */
 
 #ifdef AUTH_LDAP
-static const char rcsid[] = "$Id: auth_ldap.c,v 1.16 2003/10/23 09:43:50 chris Exp $";
+static const char rcsid[] = "$Id$";
 
 #include <sys/types.h> /* BSD needs this here, apparently. */
 
+/* Some of the LDAP APIs we're using are deprecated, sadly. Without this they
+ * don't get defined in the header files. */
+#define LDAP_DEPRECATED 1
+
 #include <lber.h>
 #include <ldap.h>
 #include <pwd.h>
