@@ -2,7 +2,21 @@
  * auth_mysql.c:
  * Authenticate users against a MySQL database.
  *
- * Copyright (c) 2001 Chris Lightfoot. All rights reserved.
+ * Copyright (c) 2001 Chris Lightfoot.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
 
@@ -11,7 +25,7 @@
 #endif /* HAVE_CONFIG_H */
 
 #ifdef AUTH_MYSQL
-static const char rcsid[] = "$Id: auth_mysql.c,v 1.54 2003/07/14 23:31:20 chris Exp $";
+static const char rcsid[] = "$Id$";
 
 #include <sys/types.h> /* BSD needs this here, apparently. */
 
@@ -44,7 +58,7 @@ static const char rcsid[] = "$Id: auth_mysql.c,v 1.54 2003/07/14 23:31:20 chris
  *  [2] unix user
  *  [3] mailbox type
  */
-char *user_pass_query_template =
+static char *user_pass_query_template =
     "SELECT concat(domain.path, '/', popbox.mbox_name), popbox.password_hash, "
             "domain.unix_user, 'bsd' "
       "FROM popbox, domain "
@@ -52,7 +66,7 @@ char *user_pass_query_template =
        "AND popbox.domain_name = '$(domain)' "
        "AND popbox.domain_name = domain.domain_name";
        
-char *apop_query_template =
+static char *apop_query_template =
     "SELECT concat(domain.path, '/', popbox.mbox_name), popbox.password_hash, "
             "domain.unix_user, 'bsd' "
       "FROM popbox, domain "
@@ -60,20 +74,20 @@ char *apop_query_template =
        "AND popbox.domain_name = '$(domain)' "
        "AND popbox.domain_name = domain.domain_name";
 
-char *onlogin_query_template = NULL;
+static char *onlogin_query_template = NULL;
 
 /* GID used to access mail spool (if any). */
-int use_gid;
-gid_t mail_gid;
+static int use_gid;
+static gid_t mail_gid;
 
 static char *substitute_query_params(const char *temp, const char *user, const char *local_part, const char *domain, const char *clienthost, const char *serverhost);
 
 /*
  * Connection to the MySQL server.
  */
-MYSQL *mysql = NULL;
-tokens mysql_servers;
-char mysql_driver_active = 0;
+static MYSQL *mysql = NULL;
+static tokens mysql_servers;
+static char mysql_driver_active = 0;
 
 /* get_mysql_server:
  * If we are not currently connected to a MySQL server, or if the current MySQL
@@ -85,6 +99,7 @@ static int get_mysql_server(void) {
     static MYSQL mysql_handle;
     char *password;
     unsigned int timeout;
+    my_bool want_reconnect = 0;
 
     if (mysql && mysql_ping(mysql) == 0)
         /* The current server is up and running. */
@@ -113,6 +128,9 @@ static int get_mysql_server(void) {
         timeout = 5;
         mysql_options(mysql, MYSQL_OPT_CONNECT_TIMEOUT, (char*)&(timeout));
 
+	/* We do not want automatic reconnect to happen. */
+	mysql_options(mysql, MYSQL_OPT_RECONNECT, &want_reconnect);
+
         if (mysql_real_connect(mysql, mysql_servers->toks[n],
                 config_get_string("auth-mysql-username"),
                 password,
@@ -267,7 +285,7 @@ authcontext auth_mysql_new_apop(const char *name, const char *local_part, const
 
                 /* User was not lying (about her password) */
                 if (!parse_uid((const char*)row[2], &uid)) {
-                    log_print(LOG_ERR, _("auth_mysql_new_apop: unix user `%s' for %s does not make sense"), row[3], who);
+                    log_print(LOG_ERR, _("auth_mysql_new_apop: unix user `%s' for %s does not make sense"), row[2], who);
                     break;
                 }
 
@@ -367,7 +385,7 @@ authcontext auth_mysql_new_user_pass(const char *user, const char *local_part, c
                 }
 
                 if (!parse_uid((const char*)row[2], &uid)) {
-                    log_print(LOG_ERR, _("auth_mysql_new_user_pass: unix user `%s' for %s does not make sense"), row[3], who);
+                    log_print(LOG_ERR, _("auth_mysql_new_user_pass: unix user `%s' for %s does not make sense"), row[2], who);
                     break;
                 }
 
