@@ -2,6 +2,20 @@
  * auth_other.c:
  * authenticate using an external program
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *
  * We send a number of variables in the form
  *
  *  key\0value\0
@@ -36,7 +50,7 @@
  *
  * The total length of the data sent or received will not exceed 4096 bytes.
  *
- * Copyright (c) 2001 Chris Lightfoot. All rights reserved.
+ * Copyright (c) 2001 Chris Lightfoot.
  *
  */
 
@@ -45,7 +59,7 @@
 #endif /* HAVE_CONFIG_H */
 
 #ifdef AUTH_OTHER
-static const char rcsid[] = "$Id: auth_other.c,v 1.26 2003/07/14 23:31:20 chris Exp $";
+static const char rcsid[] = "$Id$";
 
 #include <sys/types.h>
 
