@@ -2,7 +2,21 @@
  * auth_pam.c:
  * authenticate using Pluggable Authentication Modules
  *
- * Copyright (c) 2001 Chris Lightfoot. All rights reserved.
+ * Copyright (c) 2001 Chris Lightfoot.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
 
@@ -11,7 +25,7 @@
 #endif /* HAVE_CONFIG_H */
 
 #ifdef AUTH_PAM
-static const char rcsid[] = "$Id: auth_pam.c,v 1.29 2003/09/27 15:33:57 chris Exp $";
+static const char rcsid[] = "$Id$";
 
 #include <sys/types.h> /* BSD needs this here, it seems. */
 
