@@ -4,8 +4,21 @@
  *
  * XXX should have the connection retrying behaviour of auth-mysql.
  *
- * Copyright (c) 2003 Chris Lightfoot, Stephen White. All rights reserved.
+ * Copyright (c) 2003 Chris Lightfoot, Stephen White.
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -13,7 +26,7 @@
 #endif /* HAVE_CONFIG_H */
 
 #ifdef AUTH_PGSQL
-static const char rcsid[] = "$Id: auth_pgsql.c,v 1.1 2003/07/14 23:31:20 chris Exp $";
+static const char rcsid[] = "$Id$";
 
 #include <sys/types.h> /* BSD needs this here, apparently. */
 
@@ -105,7 +118,7 @@ size_t PQescapeStringLocal(char *to, const char *from, size_t length) {
  *  [2] unix user
  *  [3] mailbox type
  */
-char *user_pass_query_template =
+static char *user_pass_query_template =
     "SELECT domain.path || '/'  || popbox.mbox_name, popbox.password_hash, "
             "domain.unix_user, 'bsd' "
       "FROM popbox, domain "
@@ -113,7 +126,7 @@ char *user_pass_query_template =
        "AND popbox.domain_name = '$(domain)' "
        "AND popbox.domain_name = domain.domain_name";
        
-char *apop_query_template =
+static char *apop_query_template =
     "SELECT domain.path || '/' || popbox.mbox_name, popbox.password_hash, "
             "domain.unix_user, 'bsd' "
       "FROM popbox, domain "
@@ -121,11 +134,11 @@ char *apop_query_template =
        "AND popbox.domain_name = '$(domain)' "
        "AND popbox.domain_name = domain.domain_name";
 
-char *onlogin_query_template = NULL;
+static char *onlogin_query_template = NULL;
 
 /* GID used to access mail spool (if any). */
-int use_gid;
-gid_t mail_gid;
+static int use_gid;
+static gid_t mail_gid;
 
 static char *substitute_query_params(const char *temp, const char *user, const char *local_part, const char *domain, const char *clienthost, const char *serverhost);
 
@@ -148,7 +161,7 @@ static void strclr(char *s) {
  * Initialise the database connection driver. Clears the config directives
  * associated with the database so that a user cannot recover them with a
  * debugger. */
-PGconn *pg_conn;
+static PGconn *pg_conn;
 
 int auth_pgsql_init(void) {
     char *username = NULL, *password = NULL, *hostname = NULL, *database = NULL, *localhost = "localhost", *s;
