@@ -2,11 +2,24 @@
  * authswitch.c:
  * authentication driver switch
  *
- * Copyright (c) 2001 Chris Lightfoot. All rights reserved.
+ * Copyright (c) 2001 Chris Lightfoot.
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-static const char rcsid[] = "$Id: authswitch.c,v 1.42 2003/11/24 19:58:28 chris Exp $";
+static const char rcsid[] = "$Id$";
 
 #ifdef HAVE_CONFIG_H
 #include "configuration.h"
@@ -19,6 +32,7 @@ static const char rcsid[] = "$Id: authswitch.c,v 1.42 2003/11/24 19:58:28 chris
 
 #include <unistd.h>
 #include <sys/types.h>
+#include <ctype.h>
 
 #ifdef AUTH_LDAP
 #include "auth_ldap.h"
@@ -52,6 +66,10 @@ static const char rcsid[] = "$Id: authswitch.c,v 1.42 2003/11/24 19:58:28 chris
 #include "auth_passwd.h"
 #endif /* AUTH_PASSWD */
 
+#ifdef AUTH_GDBM
+#include "auth_gdbm.h"
+#endif /* AUTH_GDBM */
+
 #ifdef USE_WHOSON
 #include <whoson.h>
 #endif
@@ -122,6 +140,13 @@ struct authdrv auth_drivers[] = {
             "flatfile",
             _X("Uses /etc/passwd-style flat files")},
 #endif /* AUTH_FLATFILE */
+
+#ifdef AUTH_GDBM
+        /* Authenticate against GNU dbm files. */
+        {auth_gdbm_init, auth_gdbm_new_apop, auth_gdbm_new_user_pass, NULL, auth_gdbm_postfork, auth_gdbm_close,
+            "gdbm",
+            _X("Uses GNU dbm files")},
+#endif /* AUTH_GDBM */
 };
 
 int *auth_drivers_running;
@@ -224,14 +249,17 @@ authcontext authcontext_new_apop(const char *user, const char *local_part, const
  
     /* If no local-part has been explicitly supplied, then we try to construct
      * one by splitting up the username over one of the characters listed in
-     * DOMAIN_SEPARATORS. This is distinct from the append-domain
+     * domain-separators. This is distinct from the append-domain
      * functionality, which will attempt to use the user's supplied username
      * as a local-part, with the listener domain as the domain, and the
      * strip-domain functionality, which will suppress the domain supplied by
      * the user. */
     if (!local_part && domain) {
         int n;
-        n = strcspn(user, DOMAIN_SEPARATORS);
+        char *domsep;
+        if (!(domsep = config_get_string("domain-separators")))
+            domsep = DOMAIN_SEPARATORS;
+        n = strcspn(user, domsep);
         if (n > 0 && user[n]) {
             x = xstrdup(user);
             x[n] = 0;
@@ -280,7 +308,10 @@ authcontext authcontext_new_user_pass(const char *user, const char *local_part,
     /* Maybe split local part and domain (see above). */
     if (!local_part && domain) {
         int n;
-        n = strcspn(user, DOMAIN_SEPARATORS);
+        char *domsep;
+        if (!(domsep = config_get_string("domain-separators")))
+            domsep = DOMAIN_SEPARATORS;
+        n = strcspn(user, domsep);
         if (n > 0 && user[n]) {
             x = xstrdup(user);
             x[n] = 0;
@@ -290,25 +321,26 @@ authcontext authcontext_new_user_pass(const char *user, const char *local_part,
             l = NULL;
     }
 
-    if ((a = authcache_new_user_pass(user, l, d, pass, clienthost, serverhost)))
-        return a;
-
-    for (aa = auth_drivers, aar = auth_drivers_running; aa < auth_drivers_end; ++aa, ++aar)
-        if (*aar && aa->auth_new_user_pass && (a = aa->auth_new_user_pass(user, l, d, pass, clienthost, serverhost))) {
-            a->auth = xstrdup(aa->name);
-            a->user = xstrdup(user);
-            if (!a->local_part) {
-                if (l)
-                    a->local_part = xstrdup(l);
-                else
-                    a->local_part = xstrdup(user);
+    /* Try auth cache, then actual auth drivers. */
+    if (!(a = authcache_new_user_pass(user, l, d, pass, clienthost, serverhost))) {
+        for (aa = auth_drivers, aar = auth_drivers_running; aa < auth_drivers_end; ++aa, ++aar) {
+            if (*aar && aa->auth_new_user_pass && (a = aa->auth_new_user_pass(user, l, d, pass, clienthost, serverhost))) {
+                a->auth = xstrdup(aa->name);
+                a->user = xstrdup(user);
+                if (!a->local_part) {
+                    if (l)
+                        a->local_part = xstrdup(l);
+                    else
+                        a->local_part = xstrdup(user);
+                }
+                if (!a->domain && d)
+                    a->domain = xstrdup(d);
+                authcache_save(a, user, l, d, pass, clienthost, serverhost);
+                log_print(LOG_INFO, _("authcontext_new_user_pass: began session for `%s' with %s; uid %d, gid %d"), a->user, a->auth, a->uid, a->gid);
+                break;
             }
-            if (!a->domain && d)
-                a->domain = xstrdup(d);
-            authcache_save(a, user, l, d, pass, clienthost, serverhost);
-            log_print(LOG_INFO, _("authcontext_new_user_pass: began session for `%s' with %s; uid %d, gid %d"), a->user, a->auth, a->uid, a->gid);
-            break;
         }
+    }
 
     xfree(x);
     
@@ -330,7 +362,7 @@ void authswitch_onlogin(const authcontext A, const char *clienthost, const char
 #ifdef USE_WHOSON
     char buf[128] = {0};
     /* Notify whoson server the user has logged in correctly */
-    if (wso_login(clienthost, A->user, buf, sizeof(buf)) == -1)
+    if (whoson_enable && wso_login(clienthost, A->user, buf, sizeof(buf)) == -1)
         log_print(LOG_ERR, "authswitch_onlogin: wso_login: %s", buf);
 #endif /* USE_WHOSON */
     
@@ -377,6 +409,7 @@ void authswitch_close() {
  * Fill in a new authentication context structure with the given information. */
 authcontext authcontext_new(const uid_t uid, const gid_t gid, const char *mboxdrv, const char *mailbox, const char *home) {
     authcontext a;
+    char *mp;
 
     alloc_struct(_authcontext, a);
 
@@ -385,8 +418,13 @@ authcontext authcontext_new(const uid_t uid, const gid_t gid, const char *mboxdr
 
     if (mboxdrv)
         a->mboxdrv = xstrdup(mboxdrv);
-    if (mailbox)
+    if (mailbox) {
         a->mailbox = xstrdup(mailbox);
+        if (config_get_bool("lowercase-mailbox"))
+            for (mp = a->mailbox; *mp; *mp++)
+                *mp = tolower(*mp);
+    }
+
 
     a->auth = NULL;
     a->user = NULL;
