@@ -2,10 +2,21 @@
  * mailbox.c:
  * Generic mailbox support for tpop3d.
  *
- * Copyright (c) 2001 Chris Lightfoot, Paul Makepeace. All rights reserved.
+ * Copyright (c) 2001 Chris Lightfoot, Paul Makepeace.
  *
- * $Id: mailbox.c,v 1.6 2001/10/14 01:13:38 chris Exp $
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
  *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -18,16 +29,15 @@
 #include <unistd.h>
 
 #include "authswitch.h"
+#include "config.h"
 #include "mailbox.h"
-#include "stringmap.h"
 #include "tokenise.h"
 #include "util.h"
 
 /* mbox_drivers:
  * References various mailbox drivers, New ones should be added as below. Note
  * that the first driver in this list will be used if mailbox_new is called
- * with a NULL mailspool type, so it should be a sensible default.
- */
+ * with a NULL mailspool type, so it should be a sensible default. */
 #define _X(String) (String)
 
 struct mboxdrv mbox_drivers[] = {
@@ -59,8 +69,7 @@ struct mboxdrv mbox_drivers[] = {
 #define mbox_drivers_end    mbox_drivers + NUM_MBOX_DRIVERS
 
 /* mailbox_describe:
- * Describe available mailbox drivers.
- */
+ * Describe available mailbox drivers. */
 void mailbox_describe(FILE *fp) {
     const struct mboxdrv *mr;
     fprintf(fp, _("Available mailbox drivers:\n\n"));
@@ -72,8 +81,7 @@ void mailbox_describe(FILE *fp) {
 
 /* mailbox_new:
  * Create a new mailspool of the specified type, or a default type if the
- * passed value is NULL.
- */
+ * passed value is NULL. */
 mailbox mailbox_new(const char *filename, const char *type) {
     struct mboxdrv *mr;
 
@@ -82,7 +90,7 @@ mailbox mailbox_new(const char *filename, const char *type) {
     for (mr = mbox_drivers; mr < mbox_drivers_end; ++mr)
         if (strcmp(type, mr->name) == 0) return mr->m_new(filename);
     
-    print_log(LOG_ERR, _("mailbox_new(%s): request for unknown mailbox type %s"), filename, type);
+    log_print(LOG_ERR, _("mailbox_new(%s): request for unknown mailbox type %s"), filename, type);
     return MBOX_NOENT;
 }
 
@@ -91,8 +99,7 @@ mailbox mailbox_new(const char *filename, const char *type) {
  * mailspool... the terminology is from C++ so it doesn't have to be logical).
  *
  * Note that this does `generic' deletion; there should be specific
- * destructors for each type of mailbox.
- */
+ * destructors for each type of mailbox. */
 void mailbox_delete(mailbox m) {
     if (!m) return;
     if (m->index) {
@@ -106,8 +113,7 @@ void mailbox_delete(mailbox m) {
 }
 
 /* mailbox_add_indexpoint:
- * Add an indexpoint to a mailbox.
- */
+ * Add an indexpoint to a mailbox. */
 void mailbox_add_indexpoint(mailbox m, const struct indexpoint *i) {
     if (m->num == m->size) {
         m->index = xrealloc(m->index, m->size * sizeof(struct indexpoint) * 2);
@@ -117,58 +123,73 @@ void mailbox_add_indexpoint(mailbox m, const struct indexpoint *i) {
 }
 
 /* emptymbox_new:
- * New empty mailbox.
- */
+ * New empty mailbox. */
 mailbox emptymbox_new(const char *unused) {
     mailbox M;
-    M = xcalloc(1, sizeof *M);
+
+    alloc_struct(_mailbox, M);
+    
     if (!M) return NULL;
 
     M->delete = mailbox_delete;                 /* generic destructor */
     M->apply_changes = emptymbox_apply_changes;
-    M->send_message = NULL;                     /* should never be called */
+    M->sendmessage = NULL;                     /* should never be called */
 
-    M->name = strdup(_("[empty mailbox]"));
+    M->name = xstrdup(_("[empty mailbox]"));
     M->index = NULL;
 
     return M;
 }
 
 /* emptymbox_apply_changes:
- * Null function for empty mailbox.
- */
+ * Null function for empty mailbox. */
 int emptymbox_apply_changes(mailbox M) {
     return 1;
 }
 
 /* try_mailbox_locations:
- * Helper function for find_mailbox.
- */
-mailbox try_mailbox_locations(const char *specs, const char *user, const char *domain, const char *home) {
-    tokens t = tokens_new(specs, " \t");
+ * Helper function for find_mailbox. */
+static mailbox try_mailbox_locations(const char *specs, authcontext a) {
+    tokens t;
     mailbox m = NULL;
     int i;
 
-    if (!t) return NULL;
+    if (!(t = tokens_new(specs, " \t"))) return NULL;
     
     for (i = 0; i < t->num; ++i) {
-        char *str = t->toks[i], *mdrv = NULL, *subspec, *path;
+        char *str, *mdrv = NULL, *subspec, *path;
         struct sverr err;
 
+        str = t->toks[i];
+
         subspec = strchr(str, ':');
         if (subspec) {
             mdrv = str;
             *subspec++ = 0;
-        } else subspec = str;
+        } else {
+            subspec = str;
+            mdrv = mbox_drivers[0].name;
+        }
+
+        path = substitute_variables(subspec, &err, 4, "user", a->user, "local_part", a->local_part, "domain", a->domain, "home", a->home);
 
-        path = substitute_variables(subspec, &err, 3, "user", user, "domain", domain, "home", home);
         if (!path)
             /* Some sort of syntax error. */
-            print_log(LOG_ERR, _("try_mailbox_locations: %s near `%.16s'"), err.msg, subspec + err.offset);
+            log_print(LOG_ERR, _("try_mailbox_locations: %s near `%.16s'"), err.msg, subspec + err.offset);
         else {
             m = mailbox_new(path, mdrv);
+            /* Return in case of error ... */
+            if (!m) {
+                xfree(path);
+                break;
+            }
+            /* ... or if we found the mailspool. */
+            if(m != MBOX_NOENT) {
+                a->mboxdrv = strdup(mdrv);
+                a->mailbox = path;
+                break;
+            }
             xfree(path);
-            if (!m || m != MBOX_NOENT) break; /* Return in case of error or if we found the mailspool. */
         }
     }
     
@@ -191,25 +212,22 @@ mailbox try_mailbox_locations(const char *specs, const char *user, const char *d
  * mailbox names, both exist, and the former is locked. It is important that
  * the view of the mailbox presented to the user is consistent, so a failure
  * to lock a given mailspool must not cause the program to go off and use a
- * different one.
- */
-extern stringmap config;
-
+ * different one. */
 mailbox find_mailbox(authcontext a) {
     mailbox m = MBOX_NOENT;
     char *buffer;
-    item *I;
+    char *s;
  
     /* Try the driver-specific config option. */
     buffer = xmalloc(strlen("auth--mailbox") + strlen(a->auth) + 1);
     sprintf(buffer, "auth-%s-mailbox", a->auth);
-    if ((I = stringmap_find(config, buffer)))
-        m = try_mailbox_locations(I->v, a->user, a->domain, a->home);
+    if ((s = config_get_string(buffer)))
+        m = try_mailbox_locations(s, a);
     xfree(buffer);
 
     /* Then the global one. */
-    if (m == MBOX_NOENT && (I = stringmap_find(config, "mailbox")))
-        m = try_mailbox_locations(I->v, a->user, a->domain, a->home);
+    if (m == MBOX_NOENT && (s = config_get_string("mailbox")))
+        m = try_mailbox_locations(s, a);
     
 #ifdef MAILSPOOL_DIR
     /* Then the compiled-in default. */
@@ -224,7 +242,7 @@ mailbox find_mailbox(authcontext a) {
     /* No good. Give the user an empty mailbox. */
     if (m == MBOX_NOENT) {
         m = emptymbox_new(NULL);
-        print_log(LOG_WARNING, _("find_mailbox: using empty mailbox for user %s"), a->user);
+        log_print(LOG_WARNING, _("find_mailbox: using empty mailbox for user %s"), a->user);
     }
     
     return m;
