@@ -24,11 +24,23 @@
  * (Historically, tpop3d supported only BSD mailspools, so the terminology is
  * a little confused....)
  *
- * $Id: mailbox.h,v 1.2 2001/07/08 16:57:09 chris Exp $
+ * $Id$
  * 
  * Copyright (c) 2001 Chris Lightfoot, Paul Makepeace (realprogrammers.com).
- * All rights reserved.
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
  
 #ifndef __MAILBOX_H_ /* include guard */
@@ -43,46 +55,47 @@
 
 #include <stdio.h>
 
+/* Ugh. Forward references. Should fix. */
+struct _connection;
+
 /* struct indexpoint:
  * Individual messages which appear as files on FS for maildir or sections of
- * a file in a BSD mailspool.
- */
+ * a file in a BSD mailspool. */
 struct indexpoint {
     char    *filename;
-    size_t  offset, length, msglength;  /* Offsets and length for mailboxes.    */
-    time_t  mtime;                      /* Modification time used for maildirs. */
+    size_t  offset, length, msglength;  /* Offsets, length for mailspools.  */
+    time_t  mtime;                      /* Modified time used for maildirs. */
     char    deleted;
     unsigned char hash[16];
 };
 
 /* mailbox:
- * Generic object representing a store of messages.
- */
+ * Generic object representing a store of messages. */
 typedef struct _mailbox *mailbox;
 
 struct _mailbox {
-    char *name;                 /* Spool filename or maildir directory name.    */
-    int fd;                     /* File descriptor for open mailspool.          */
-    char isempty;               /* Boolean for mailspool.                       */
-    struct stat st;             /* stat(2) buffer for file/directory.           */
-    struct indexpoint *index;   /* Array of message lengths/offsets.            */
-    int num, size;              /* Number of messages, and space allocated.     */
-    int numdeleted;             /* Number of messages deleted by client.        */
+    char *name;                 /* Spool filename or maildir directory name. */
+    int fd;                     /* File descriptor for open mailspool.       */
+    char isempty;               /* Boolean for mailspool.                    */
+    struct stat st;             /* stat(2) buffer for file/directory.        */
+    struct indexpoint *index;   /* Array of message lengths/offsets.         */
+    int num, size;              /* Number of messages, and space allocated.  */
+    int numdeleted;             /* Number of messages deleted by client.     */
+    int totalsize;              /* Sum of message sizes.                     */
+    int sizedeleted;            /* Sum of deleted message sizes.             */
 
     /* function pointers for pseudo OO-ness */
     void    (*delete)(mailbox m);
-    int     (*send_message)(mailbox m, int sck, const int i, int n);
+    int     (*sendmessage)(mailbox m, struct _connection* c, const int i, int n);
     int     (*apply_changes)(mailbox m);
 };
 
 /* Return code from a mailbox constructor used to indicate non-presence of the
- * mailbox.
- */
+ * mailbox. */
 #define MBOX_NOENT      ((mailbox)-1)
 
 /* struct mboxdrv:
- * Structure for describing alternate mailspool drivers.
- */
+ * Structure for describing alternate mailspool drivers. */
 struct mboxdrv {
     char *name;
     char *description;
@@ -105,7 +118,7 @@ int     emptymbox_apply_changes(mailbox m);
 mailbox mailspool_new_from_file(const char *filename);
 void    mailspool_delete(mailbox m);
 int     mailspool_build_index(mailbox m, char *filemem);
-int     mailspool_send_message(mailbox m, int sck, const int i, int n);
+int     mailspool_sendmessage(mailbox m, struct _connection* c, const int i, int n);
 int     mailspool_apply_changes(mailbox m);
 
 /* How long we wait between trying to lock the mailspool */
@@ -119,7 +132,7 @@ int     mailspool_apply_changes(mailbox m);
 #ifdef MBOX_MAILDIR
 mailbox maildir_new(const char *filename);
 void    maildir_delete(mailbox m);
-int     maildir_send_message(mailbox m, int sck, const int i, int n);
+int     maildir_sendmessage(mailbox m, struct _connection* c, const int i, int n);
 int     maildir_apply_changes(mailbox m);
 #endif /* MBOX_MAILDIR */
 
