@@ -17,8 +17,21 @@
  * A further option allows caches of message offsets in a mailspool to be
  * made, reducing the time needed to open a mailspool.
  *
- * Copyright (c) 2001 Chris Lightfoot. All rights reserved.
+ * Copyright (c) 2001 Chris Lightfoot.
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -26,7 +39,7 @@
 #endif /* HAVE_CONFIG_H */
 
 #ifdef MBOX_BSD
-static const char rcsid[] = "$Id: mailspool.c,v 1.50 2003/11/06 01:19:27 chris Exp $";
+static const char rcsid[] = "$Id$";
 
 #include <errno.h>
 #include <fcntl.h>
@@ -60,6 +73,10 @@ int mailspool_save_index(mailbox m);
 int mailspool_load_index(mailbox m);
 #endif /* MBOX_BSD_SAVE_INDICES */
 
+/* Don't try to dotfile-lock a mailspool, even if support for doing so is
+ * available. */
+int mailspool_no_dotfile_locking;
+
 /* file_unlock FD FILENAME
  * Unlock a mailspool file using the open FD and given FILENAME. Returns 1 on
  * success or 0 on failure. */
@@ -74,7 +91,8 @@ int file_unlock(const int fd, const char *name) {
 #endif
 
 #ifdef WITH_DOTFILE_LOCKING
-    if (name && dotfile_unlock(name) == -1) r = 0;
+    if (!mailspool_no_dotfile_locking
+        && name && dotfile_unlock(name) == -1) r = 0;
 #endif
 
     return r;
@@ -97,8 +115,10 @@ int file_lock(const int fd, const char *name) {
     else l_flock = 1;
 #endif
 #ifdef WITH_DOTFILE_LOCKING
-    if (dotfile_lock(name) == -1) goto fail;
-    else l_dotfile = 1;
+    if (!mailspool_no_dotfile_locking) {
+        if (dotfile_lock(name) == -1) goto fail;
+        else l_dotfile = 1;
+    }
 #endif
 #ifdef WITH_CCLIENT_LOCKING
     if (cclient_steal_lock(fd) == -1) goto fail;
@@ -114,7 +134,8 @@ fail:
     if (l_flock) flock_unlock(fd);
 #endif
 #ifdef WITH_DOTFILE_LOCKING
-    if (l_dotfile) dotfile_unlock(name);
+    if (!mailspool_no_dotfile_locking
+        && l_dotfile) dotfile_unlock(name);
 #endif
     
     return 0;
@@ -750,6 +771,8 @@ int mailspool_load_index(mailbox m) {
     }
 
     if (!feof(fp)) {
+        --m->num;
+
         log_print(LOG_WARNING, _("mailspool_load_index(%s): index exists, but has some stale or corrupt data"), indexfile);
         goto fail;
     }
