@@ -2,12 +2,25 @@
  * main.c:
  * Entry point, initialisation code and main loop for pop3 server.
  *
- * Copyright (c) 2001 Chris Lightfoot. All rights reserved.
+ * Copyright (c) 2001 Chris Lightfoot.
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-static const char copyright[] = "$Copyright: (c) 2001 Chris Lightfoot. All rights reserved. $";
-static const char rcsid[] = "$Id: main.c,v 1.93 2003/11/24 19:58:28 chris Exp $";
+static const char copyright[] = "$Copyright: (c) 2001 Chris Lightfoot. $";
+static const char rcsid[] = "$Id$";
 
 #ifdef HAVE_CONFIG_H
 #include "configuration.h"
@@ -340,6 +353,10 @@ char optstring[] = "+hdvf:p:"
 extern int mailspool_save_indices;  /* in mailspool.c */
 #endif
 
+#ifdef MBOX_BSD
+extern int mailspool_no_dotfile_locking;    /* in mailspool.c */
+#endif
+
 int main(int argc, char **argv, char **envp) {
     int nodaemon = 0;
     char *configfile = CONFIG_DIR"/tpop3d.conf", *s;
@@ -427,6 +444,12 @@ int main(int argc, char **argv, char **envp) {
     }
 #endif
 
+    /* Should we skip dotfile-locking. */
+#ifdef MBOX_BSD
+    if (config_get_bool("mailspool-no-dotfile-locking"))
+        mailspool_no_dotfile_locking = 1;
+#endif
+
     /* We may have been compiled with TCP wrappers support. */
 #ifdef USE_TCP_WRAPPERS
     if (!(tcpwrappersname = config_get_string("tcp-wrappers-name")))
@@ -445,6 +468,8 @@ retry_pid_file:
                 pid_t pid;
                 switch (read_pid_file(pidfile, &pid)) {
                     case pid_file_success:
+                        if (getpid() == pid)
+                            break;
                         if (kill(pid, 0)) {
                             log_print(LOG_ERR, _("%s: stale PID file; removing it"), pidfile);
                             if (unlink(pidfile) == -1) {
@@ -453,7 +478,7 @@ retry_pid_file:
                             } else goto retry_pid_file; /* harmful? */
                             
                         } else {
-                            log_print(LOG_ERR, _("%s: tpop3d already running, with process ID %d; exiting"), (int)pid);
+                            log_print(LOG_ERR, _("tpop3d already running, with process ID %d; exiting"), (int)pid);
                             return 1;
                         }
                         break;
