@@ -121,13 +121,16 @@ static void remove_connection(connection c) {
 /* listeners_pre_select:
  * Called before the main select(2) so listening sockets can be polled. */
 static void listeners_pre_select(int *n, struct pollfd *pfds) {
+    int i = *n;
     item *t;
     vector_iterate(listeners, t) {
         int s = ((listener)t->v)->s;
-       pfds[s].fd = s;
-       pfds[s].events |= POLLIN;
-        if (s > *n) *n = s;
+	pfds[i].fd = s;
+	pfds[i].events |= POLLIN;
+	((listener)t->v)->s_index = i;
+	i++;
     }
+    *n = i;
 }
 
 /* listeners_post_select:
@@ -137,7 +140,7 @@ static void listeners_post_select(struct pollfd *pfds) {
     item *t;
     vector_iterate(listeners, t) {
         listener L = (listener)t->v;
-       if (pfds[L->s].revents & (POLLIN | POLLHUP)) {
+       if (pfds[L->s_index].revents & (POLLIN | POLLHUP)) {
             struct sockaddr_in sin, sinlocal;
             size_t l = sizeof(sin);
             static int tcp_send_buf = -1;
@@ -549,6 +552,8 @@ void net_loop(void) {
     extern int child_died_signal;
     sigset_t chmask;
     struct pollfd *pfds;
+    int max_listeners;
+    item *t;
     
     sigemptyset(&chmask);
     sigaddset(&chmask, SIGCHLD);
@@ -557,15 +562,21 @@ void net_loop(void) {
     max_connections = 2 * max_running_children;
     connections = (connection*)xcalloc(max_connections, sizeof(connection*));
 
-    pfds = xmalloc(max_connections * sizeof *pfds);
+    /* find out number of listeners */
+    max_listeners = 0;
+    vector_iterate(listeners, t)
+	    max_listeners++;
+
+    pfds = xmalloc((max_listeners + max_connections) * sizeof *pfds);
 
     log_print(LOG_INFO, _("net_loop: tpop3d version %s successfully started"), TPOP3D_VERSION);
     
     /* Main select() loop */
     while (!foad) {
-        int n = 0, e, i;
+        int n = 0; /* number of pfds elements in use */
+        int e, i;
 
-        for (i = 0; i < max_connections; ++i) {
+        for (i = 0; i < (max_listeners + max_connections); ++i) {
             pfds[i].fd = -1;
             pfds[i].events = pfds[i].revents = 0;
         }
@@ -574,7 +585,7 @@ void net_loop(void) {
 
         connections_pre_select(&n, pfds);
 
-        e = poll(pfds, n + 1, 1000 /* must be smaller than timeout */);
+        e = poll(pfds, n, 1000 /* must be smaller than timeout */);
         if (e == -1 && errno != EINTR) {
             log_print(LOG_WARNING, "net_loop: poll: %m");
         } else if (e >= 0) {
