@@ -2,11 +2,24 @@
  * pop3.c:
  * implementation of rfc1939 POP3
  *
- * Copyright (c) 2000 Chris Lightfoot. All rights reserved.
+ * Copyright (c) 2000 Chris Lightfoot.
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-static const char rcsid[] = "$Id: pop3.c,v 1.57 2003/11/06 01:19:27 chris Exp $";
+static const char rcsid[] = "$Id$";
 
 #ifdef HAVE_CONFIG_H
 #include "configuration.h"
@@ -22,10 +35,12 @@ static const char rcsid[] = "$Id: pop3.c,v 1.57 2003/11/06 01:19:27 chris Exp $"
 #include <arpa/inet.h>
 #include <sys/socket.h>
 #include <sys/types.h>
+#include <ctype.h>
 
 #include "authswitch.h"
 #include "connection.h"
 #include "util.h"
+#include "config.h"
 
 extern int verbose;
 
@@ -70,6 +85,7 @@ static enum connection_action do_capa(connection c) {
  * USER command; supply username of client. Returns 1 on success or 0 on
  * failure. */
 static int do_user(connection c, const pop3command p) {
+    char *up;
     if (p->toks->num != 2) {
         connection_sendresponse(c, 0, _("No, that's not right."));
         return 0;
@@ -78,6 +94,9 @@ static int do_user(connection c, const pop3command p) {
         return 0;
     } else {
         c->user = xstrdup((char*)p->toks->toks[1]);
+        if (config_get_bool("lowercase-user"))
+            for (up = c->user; *up; *up++)
+                *up = tolower(*up);
         if (!c->user)
 #ifndef NO_SNIDE_COMMENTS
             connection_sendresponse(c, 0, _("Tell me your name, knave!"));
@@ -160,8 +179,11 @@ static enum connection_action do_apop(connection c, const pop3command p) {
     /* Maybe retry authentication with an added or removed domain name. */
     if (!c->a && (strip_domain || append_domain)) {
         int n, len;
+        char *domsep;
         len = strlen(name);
-        n = strcspn(name, DOMAIN_SEPARATORS);
+        if (!(domsep = config_get_string("domain-separators")))
+            domsep = DOMAIN_SEPARATORS;
+        n = strcspn(name, domsep);
         if (append_domain && c->domain && n == len)
             /* OK, if we have a domain name, try appending that. */
             c->a = authcontext_new_apop(name, name, c->domain, c->timestamp, digest, c->remote_ip, c->local_ip);
@@ -246,30 +268,99 @@ void do_list(connection c, const int msg_num) {
 /* do_uidl CONNECTION MSGNUM
  * UIDL command: MSGNUM is the argument or -1 if none was specified. */
 static void do_uidl(connection c, const int msg_num) {
-    /* It isn't guaranteed that these IDs are unique; it is likely, though.
-     * See RFC1939. */
     if (msg_num != -1) {
         struct indexpoint *curmsg;
         curmsg = c->m->index + msg_num;
         if (curmsg->deleted)
             connection_sendresponse(c, 0, _("That message is no more."));
         else {
-            char response[64] = {0};
-            snprintf(response, 63, "%d %s", 1 + msg_num, hex_digest(curmsg->hash));
+            char *idstyle;
+            char response[128] = {0};
+            if (!(idstyle = config_get_string("uidl-style")))
+                idstyle = "tpop3d";
+
+            /* qmail-pop3d style unique-ids */
+            if(   strcmp(idstyle, "qmail") == 0
+               && strcmp(c->a->mboxdrv, "maildir") == 0)
+            {
+                /* qmail-pop3d creates IDs by printing the filename of the message.
+                 * We have to care about suffixes like ":<something>", that e.g.
+                 * qmail-pop3d appends to messages it has read, because in uidl lists
+                 * these suffixes don't get printed.
+                 * Additionally, qmail-pop3d doesn't seem to limit the length of
+                 * unique-ids, violating RFC1939. We cut of uinique-id listings at 127
+                 * characters here, for the sake of compatibility to qmail, with the
+                 * penalty of slight RFC-ignorance. It's very unlikely, but we're still
+                 * not compatible to qmail-pop3d uinique-ids for very long filenames.
+                 *
+                 * So there are two steps we have to take:
+                 * First we print the complete filename, without any leading directory
+                 * parts. This will give uniform IDs even with maildir recursion.
+                 * Then we omit the suffixes by zero-terminating the string at the
+                 * first ':' we find, if any. (That's what qmail-pop3d does too.)
+                 */
+                snprintf(response, 127, "%d %s", 1 + msg_num, 1 + strrchr(curmsg->filename, '/'));
+                response[strcspn(response, ":")] = '\0';
+
+            /*
+             * any unique-id format other than tpop3ds native should go here
+             */
+
+            /* "tpop3d" and fallback for unknown uidl formats */
+            } else {
+                if(strcmp(idstyle, "tpop3d") != 0)
+                    log_print(LOG_WARNING, _("do_uidl: '%s' UIDLs not implemented, or not supported with '%s' mailbox, using fallback."), idstyle, c->a->mboxdrv);
+
+                /* It isn't guaranteed that these IDs are unique; it is likely, though.
+                 * See RFC1939. */
+                snprintf(response, 63, "%d %s", 1 + msg_num, hex_digest(curmsg->hash));
+            }
+
             connection_sendresponse(c, 1, response);
         }
     } else {
         struct indexpoint *m;
         int nn = 0;
+        char *idstyle;
+
         if (!(connection_sendresponse(c, 1, _("ID list follows:"))))
             return;
-        for (m = c->m->index; m < c->m->index + c->m->num; ++m) {
-            if (!m->deleted) {
-                char response[64] = {0};
-                snprintf(response, 63, "%d %s", 1 + m - c->m->index, hex_digest(m->hash));
-                if (!connection_sendline(c, response))
-                    return;
-                ++nn;
+
+        if (!(idstyle = config_get_string("uidl-style")))
+            idstyle = "tpop3d";
+
+        if(   strcmp(idstyle, "qmail") == 0
+           && strcmp(c->a->mboxdrv, "maildir") == 0)
+        {
+            char response[128] = {0};
+            for (m = c->m->index; m < c->m->index + c->m->num; ++m) {
+                if (!m->deleted) {
+                    snprintf(response, 127, "%d %s", 1 + m - c->m->index, 1 + strrchr(m->filename, '/'));
+                    response[strcspn(response, ":")] = '\0';
+                    if (!connection_sendline(c, response))
+                        return;
+                    ++nn;
+                }
+            }
+
+        /*
+         * any unique-id format other than tpop3ds native should go here
+         */
+
+        /* "tpop3d" and fallback for unknown uidl formats */
+        } else {
+            char response[64] = {0};
+
+            if(strcmp(idstyle, "tpop3d") != 0)
+                log_print(LOG_WARNING, _("do_uidl: '%s' UIDLs not implemented, or not supported with '%s' mailbox, using fallback."), idstyle, c->a->mboxdrv);
+
+            for (m = c->m->index; m < c->m->index + c->m->num; ++m) {
+                if (!m->deleted) {
+                    snprintf(response, 63, "%d %s", 1 + m - c->m->index, hex_digest(m->hash));
+                    if (!connection_sendline(c, response))
+                        return;
+                    ++nn;
+                }
             }
         }
         connection_sendline(c, ".");
@@ -295,10 +386,10 @@ static enum connection_action do_retr(connection c, const int msg_num) {
             if (verbose)
                 log_print(LOG_DEBUG, _("do_retr: client %s: sending message %d (%d bytes)"),
                         c->idstr, msg_num + 1, (int)curmsg->msglength);
-        
+            
             if ((n = c->m->sendmessage(c->m, c, msg_num, -1)) == -2)
                 return close_connection;
-            
+
             /* That might have taken a long time. */
             c->idlesince = time(NULL);
             if (verbose) {
@@ -421,7 +512,7 @@ enum connection_action connection_do(connection c, const pop3command p) {
                     } else {
                         /* Otherwise we've issued an OK response but haven't
                          * managed to negotiate a TLS connection. */
-                        log_print(LOG_INFO, _("connection_do: client %s: tried and failed to negotiate TLS connection; closing connection"), c->idstr);
+                        log_print(LOG_NOTICE, _("connection_do: client %s: tried and failed to negotiate TLS connection; closing connection"), c->idstr);
                         return close_connection;
                     }
                 }
@@ -450,8 +541,11 @@ enum connection_action connection_do(connection c, const pop3command p) {
             /* Maybe retry authentication with an added or removed domain name. */
             if (!c->a && (append_domain || strip_domain)) {
                 int n, len;
+                char *domsep;
                 len = strlen(c->user);
-                n = strcspn(c->user, DOMAIN_SEPARATORS);
+                if (!(domsep = config_get_string("domain-separators")))
+                    domsep = DOMAIN_SEPARATORS;
+                n = strcspn(c->user, domsep);
                 if (append_domain && c->domain && n == len)
                     /* OK, if we have a domain name, try appending that. */
                     c->a = authcontext_new_user_pass(c->user, c->user, c->domain, c->pass, c->remote_ip, c->local_ip);
@@ -583,6 +677,9 @@ enum connection_action connection_do(connection c, const pop3command p) {
         }
         
         switch (p->cmd) {
+            case CAPA:
+                return do_capa(c);
+
             case LIST:
                 do_list(c, msg_num);
                 break;
@@ -631,7 +728,7 @@ enum connection_action connection_do(connection c, const pop3command p) {
 #ifndef NO_SNIDE_COMMENTS
                 connection_sendresponse(c, 0, _("It's a bit late for that now, isn't it?"));
 #else
-                connection_sendresponse(c, 0, _("STLS command available only in AUTHORIZATION stat"));
+                connection_sendresponse(c, 0, _("STLS command available only in AUTHORIZATION state"));
 #endif
                 break;
 
