@@ -2,11 +2,24 @@
  * util.c:
  * Various utility functions for tpop3d.
  *
- * Copyright (c) 2001 Chris Lightfoot. All rights reserved.
+ * Copyright (c) 2001 Chris Lightfoot.
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-static const char rcsid[] = "$Id: util.c,v 1.24 2003/11/06 01:19:27 chris Exp $";
+static const char rcsid[] = "$Id$";
 
 #ifdef HAVE_CONFIG_H
 #include "configuration.h"
@@ -47,6 +60,8 @@ ssize_t xwrite(int fd, const void *buf, size_t count) {
 /* daemon:
  * Become a daemon. From `The Unix Programming FAQ', Andrew Gierth et al. */
 int daemon(int nochdir, int noclose) {
+    pid_t parent_pid = getpid();
+
     switch (fork()) {
         case 0:  break;
         case -1: return -1;
@@ -71,6 +86,12 @@ int daemon(int nochdir, int noclose) {
         dup(0); dup(0);
     }
 
+    /* wait for parent to exit, to avoid race condition
+     * with PID file handling main.c */
+    while(!kill(parent_pid,0)) {
+        usleep(10);
+    }
+
     return 0;
 }
 
