@@ -2,10 +2,21 @@
  * vector.h:
  * simple vectors
  *
- * Copyright (c) 2001 Chris Lightfoot. All rights reserved.
+ * Copyright (c) 2001 Chris Lightfoot.
  *
- * $Id: vector.h,v 1.6 2001/01/21 12:30:50 chris Exp $
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
  *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifndef __VECTOR_H_ /* include guard */
@@ -26,7 +37,7 @@ typedef struct _vector{
     size_t n, n_used;
 } *vector;
 
-vector vector_new();
+vector vector_new(void);
 void vector_delete(vector);
 void vector_delete_free(vector);
 
